module Authful
  module Api
    def self.included(base)
      base.extend ClassMethods
    end

    module ClassMethods
      def enroll(email, phone = nil)
        res = Authful.send_request(:post, "/api/users", email: email, phone: phone)
        return Authful::Response.new(res)
      end

      def get_user(token)
        res = Authful.send_request(:get, "/api/users/#{token}")
        res
      end

      def validate(token, otp)
        res = Authful.send_request(:get, "/api/users/#{token}/validate", token: otp)
        res["ok"] == 1
      end

      def send_sms(token)
        res = Authful.send_request(:get, "/api/users/#{token}/send_sms")
        res["ok"] == 1
      end

      def send_fallback_sms(token)
        res = Authful.send_request(:get, "/api/users/#{token}/fallback")
        res["ok"] == 1
      end

      def set_fallback_phone(token, phone)
        res = Authful.send_request(:patch, "/api/users/#{token}/fallback", phone: phone)
        res["error"] == nil
      end

      def reset(token, send_sms = false)
        res = Authful.send_request(:patch, "/api/users/#{token}/reset")
        return Authful::Response.new(res)
      end

      def generate_recovery_codes(token)
        res = Authful.send_request(:post, "/api/users/#{token}/recovery_codes")
        res["recovery_codes"]
      end

      def view_recovery_codes(token)
        res = Authful.send_request(:get, "/api/users/#{token}/recovery_codes")
        res["recovery_codes"]
      end

      def validate_recovery_code(token, code)
        return false if code.empty?
        res = Authful.send_request(:get, "/api/users/#{token}/recovery_codes/#{code}")
        res["ok"] == 1
      end

      def unenroll(token)
        res = Authful.send_request(:delete, "/api/users/#{token}")
        res["ok"] == 1
      end
    end
  end
end
