module Authful
  module Endpoint
    def self.included(base)
      base.extend ClassMethods
      base.class_eval do
        @config = Struct.new(:endpoint, :api_key).new
      end
    end

    module ClassMethods
      def configure(&block)
        block.call(@config)
      end

      def config
        @config
      end

      def send_request(method, path, data = {})
        if %w(get delete).include?(method.to_s)
          unless data.empty?
            path << "?" << data.map{|k,v| "#{k}=#{v}"}.join("&")
          end
          r = RestClient::Request.execute(method: method.downcase.to_sym, url: url_for(path), headers: headers, ssl_version: "TLSv1_2")

        else
          r = RestClient::Request.execute(method: method.downcase.to_sym, url: url_for(path), headers: headers, payload: data, ssl_version: "TLSv1_2")
        end
        JSON.parse(r)
      rescue
        capture_common_errors($!)
      end

      def capture_common_errors(error)
        if error.http_code == 400
          JSON.parse($!.response)
        elsif error.http_code == 401
          raise Authful::Errors::IncorrectApiToken
        elsif error.http_code == 403
          JSON.parse($!.response)
        elsif error.http_code == 404
          raise Authful::Errors::InvalidUserToken
          JSON.parse($!.response)
        elsif error.http_code == 409
          JSON.parse($!.response)
        else
          raise error
        end
      end

      def url_for(path)
        [Authful.config.endpoint, path].join("")
      end

      def headers
        {"Api-Token" => Authful.config.api_key}
      end
    end
  end
end
