module Authful
  class User
    attr_accessor :token, :email, :phone, :error, :qr_code

    def self.[](token)
      self.new(token: token)
    end

    def initialize(args)
      args.each do |k,v|
        instance_variable_set("@#{k}", v) unless v.nil?
      end
    end

    def save
      r = post("/api/users", {email: email, phone: phone})
      self.qr_code = r["qr_code"]
      self.token = r["token"]
    rescue
      if Authful::Errors::IncorrectApiToken === $!
        raise Authful::Errors::IncorrectApiToken
      elsif $!.http_code == 409 # Catch duplicate user -- already enrolled
        r = JSON.parse($!.response)
        self.token = r["token"]
      elsif $!.http_code == 400
        r = JSON.parse($!.response)
        self.error = r["error"]
        false
      else
        raise $!
      end
    end

    def self.create!(args)
      self.new(args).save
    end

    def destroy
      delete("/api/users/#{token}")["ok"] == 1
    end

    def send_sms
      get("/api/users/#{token}/send_sms")["ok"] == 1
    rescue
      if Authful::Errors::IncorrectApiToken === $!
        raise Authful::Errors::IncorrectApiToken
      elsif $!.http_code == 400
        r = JSON.parse($!.response)
        self.error = r["error"]
        false
      else
        raise $!
      end
    end

    def valid?(otp_token)
      get("/api/users/#{token}/validate?token=#{otp_token}")["ok"] == 1
    rescue
      if Authful::Errors::IncorrectApiToken === $!
        raise Authful::Errors::IncorrectApiToken
      elsif $!.http_code == 403
        r = JSON.parse($!.response)
        self.error = r["error"]
        false
      else
        raise $!
      end
    end
  end
end
